﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;

namespace FastAny
{
    class Program
    {
        static void Main(string[] args)
        {
            var random = new Random();
            for (int i = 1; i < 1000000; i++)
            {
                var itemCollection = new List<int>(i);
                int[] itemsArray = new int[i];
                for (int j = 0; j < i; j++)
                {
                    var num = random.Next();
                    itemCollection.Add(num);
                    itemsArray[j] = num;
                }

                var stopwatch = Stopwatch.StartNew();
                for (int j = 0; j < 1000000; j++)
                {
                    bool hasAny = itemCollection.Any();
                }
                stopwatch.Stop();
                Console.WriteLine(string.Format("Time to find Any() with {0} items in ICollection: {1}", i, stopwatch.ElapsedMilliseconds));

                stopwatch = Stopwatch.StartNew();
                for (int j = 0; j < 1000000; j++)
                {
                    bool hasAny = itemCollection.FastAny();
                }
                stopwatch.Stop();
                Console.WriteLine(string.Format("Time to find FastAny() with {0} items in ICollection: {1}", i, stopwatch.ElapsedMilliseconds));

                stopwatch = Stopwatch.StartNew();
                for (int j = 0; j < 1000000; j++)
                {
                    bool hasAny = itemsArray.Any();
                }
                stopwatch.Stop();
                Console.WriteLine(string.Format("Time to find Any() with {0} items in array: {1}", i, stopwatch.ElapsedMilliseconds));

                stopwatch = Stopwatch.StartNew();
                for (int j = 0; j < 1000000; j++)
                {
                    bool hasAny = itemsArray.FastAny();
                }
                stopwatch.Stop();
                Console.WriteLine(string.Format("Time to find FastAny() with {0} items in array: {1}", i, stopwatch.ElapsedMilliseconds));
                Console.WriteLine();
            }
            Console.ReadLine();
        }
    }

    static class Extensions
    {
        public static bool FastAny<TSource>(this IEnumerable<TSource> source)
        {
            if (source == null) throw new ArgumentNullException("source");

            var array = source as TSource[];
            if (array != null) return array.Length > 0;

            var coll = source as ICollection<TSource>;
            if (coll != null) return coll.Count > 0;

            using (var enumerator = source.GetEnumerator())
            {
                return enumerator.MoveNext();
            }
        }
    }
}
